@@ -26,15 +26,23 @@ echo >&2
 #
 # The mode "mbr_only" implements an alternative layout according to UEFI 2.4,
 # section 2.5.1 and table 16. No GTP, HFS+, or APM.
+# This mode produces a mountable ISO 9660 partition 1 only if variable
+# MKRESCUE_SED_PROTECTIVE is empty or set to "no".
 #
 # The mode "mbr_hfs" is like "mbr_only" but with HFS+ mentioned in APM.
 # It is still compliant to UEFI with no potentially deceiving GPT.
 # If you add xorrisofs option -part_like_isohybrid then no gap fillig APM
 # partition will emerge.
 #
+# Mode "gpt_appended" represents the same layout as "mbr_only" by GPT rather
+# than by MBR partition table. It differs from "original" by the fact that
+# option -partition_offset 16 is implied and that the first partition may
+# be used to mount the ISO 9660 filesystem. MKRESCUE_SED_PROTECTIVE is ignored,
+# because neat GPT is indicated by the existence of a Protective MBR.
+#
 # These modes avoid duplicate storing of the EFI system partition "efi.img"
 # by xorrisofs option  -e "--interval:appended_partition_${partno}:all::"
-# which is new to xorriso-1.4.3 with timestamp after 2015.12.30.175951.
+# which is new to xorriso-1.4.4.
 # If "_copy" is appended to the mode name, then the file /efi.img will
 # appear in the ISO 9660 filesystem and traditional -e "/efi.img" is used.
 #
@@ -50,9 +58,10 @@ echo >&2
 #  "mjg"         =  ESP in MBR+GPT+APM, with HFS+
 #  "mbr_only"    =  ESP in MBR, without HFS+
 #  "mbr_hfs"     =  ESP in MBR, HFS+ in APM
+#  "gpt_appended" = ESP in GPT, without HFS+
 #  $mode"_copy"  =  one of above modes, ESP in ISO and as appended partition
 #  "original"    =  pass arguments unchanged
-mode="mjg"
+mode="mbr_only"
 if test -n "$MKRESCUE_SED_MODE"
 then
   mode="$MKRESCUE_SED_MODE"
@@ -67,7 +76,7 @@ fi
 
 # Replacement for option --protective-msdos-label. Either itself or empty text.
 # If the environment variable contains the word "no", this means empty.
-protective="--protective-msdos-label"
+protective=""
 if test -n "$MKRESCUE_SED_PROTECTIVE"
 then
   if test x"$MKRESCUE_SED_PROTECTIVE" = xno
@@ -136,6 +145,13 @@ then
   echo >&2
 fi
 
+# Check for option -iso_mbr_part_type which is new in 1.4.8
+iso_mbr_part_type=
+if "$xorriso" -as mkisofs -help 2>&1 | grep iso_mbr_part_type >/dev/null
+then
+  iso_mbr_part_type="-iso_mbr_part_type 0x00"
+fi
+
 # Look for the name of the /tmp directory with the GRUB2 files.
 # It is the next argument after -r. But as default accept any /tmp/grub.*
 next_is_dir=0
@@ -194,7 +210,7 @@ then
   efi_tmp_name=grub-mkrescue-sed-efi-img.$$
   mv "$dir"/efi.img /tmp/$efi_tmp_name
   x=$(echo " $*" | sed \
-    -e "s/-efi-boot-part --efi-boot-image/-no-pad -append_partition 2 0xef \/tmp\/$efi_tmp_name/" \
+    -e "s/-efi-boot-part --efi-boot-image/$iso_mbr_part_type -no-pad -append_partition 2 0xef \/tmp\/$efi_tmp_name/" \
     -e "s/--efi-boot efi\.img/-eltorito-alt-boot -e --interval:appended_partition_2:all:: -no-emul-boot/" \
     -e "s/-hfsplus .*CoreServices\/boot.efi//" \
     -e "s/--protective-msdos-label/$protective/" \
@@ -204,7 +220,7 @@ elif test x"$mode" = xmbr_only_copy
 then
   # Exchange arguments for no-HFS MBR-only layout
   x=$(echo " $*" | sed \
-    -e "s/-efi-boot-part --efi-boot-image/-no-pad -append_partition 2 0xef \/tmp\/$(basename "$dir")\/efi.img/" \
+    -e "s/-efi-boot-part --efi-boot-image/$iso_mbr_part_type -no-pad -append_partition 2 0xef \/tmp\/$(basename "$dir")\/efi.img/" \
     -e "s/-hfsplus .*CoreServices\/boot.efi//" \
     -e "s/--protective-msdos-label/$protective/" \
      )
@@ -215,7 +231,7 @@ then
   efi_tmp_name=grub-mkrescue-sed-efi-img.$$
   mv "$dir"/efi.img /tmp/$efi_tmp_name
   x=$(echo " $*" | sed \
-    -e "s/-efi-boot-part --efi-boot-image/-no-pad -append_partition 2 0xef \/tmp\/$efi_tmp_name/" \
+    -e "s/-efi-boot-part --efi-boot-image/$iso_mbr_part_type -no-pad -append_partition 2 0xef \/tmp\/$efi_tmp_name/" \
     -e "s/--efi-boot efi\.img/-eltorito-alt-boot -e --interval:appended_partition_2:all:: -no-emul-boot/" \
     -e "s/--protective-msdos-label/$protective/" \
      )
@@ -224,10 +240,29 @@ elif test x"$mode" = xmbr_hfs_copy
 then
   # Exchange arguments for MBR and HFS+ layout
   x=$(echo " $*" | sed \
-    -e "s/-efi-boot-part --efi-boot-image/-no-pad -append_partition 2 0xef \/tmp\/$(basename "$dir")\/efi.img/" \
+    -e "s/-efi-boot-part --efi-boot-image/$iso_mbr_part_type -no-pad -append_partition 2 0xef \/tmp\/$(basename "$dir")\/efi.img/" \
     -e "s/--protective-msdos-label/$protective/" \
      )
 
+elif test x"$mode" = xgpt_appended
+then
+  # Exchange arguments for no-HFS MBR-only layout
+  efi_tmp_name=grub-mkrescue-sed-efi-img.$$
+  mv "$dir"/efi.img /tmp/$efi_tmp_name
+  x=$(echo " $*" | sed \
+    -e "s/-efi-boot-part --efi-boot-image/-no-pad -append_partition 2 0xef \/tmp\/$efi_tmp_name -appended_part_as_gpt -partition_offset 16/" \
+    -e "s/--efi-boot efi\.img/-eltorito-alt-boot -e --interval:appended_partition_2:all:: -no-emul-boot/" \
+    -e "s/-hfsplus .*CoreServices\/boot.efi//" \
+     )
+
+elif test x"$mode" = xgpt_appended_copy
+then
+  # Exchange arguments for no-HFS MBR-only layout
+  x=$(echo " $*" | sed \
+    -e "s/-efi-boot-part --efi-boot-image/-no-pad -append_partition 2 0xef \/tmp\/$(basename "$dir")\/efi.img -appended_part_as_gpt -partition_offset 16/" \
+    -e "s/-hfsplus .*CoreServices\/boot.efi//" \
+     )
+
 elif test x"$mode" = xoriginal
 then
   # Pass arguments unchanged
