@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /*
-   Copyright (c) 2009 - 2013 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2009 - 2016 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -45,7 +45,7 @@ sg_initialize()         performs global initialization of the SCSI transport
                         software components.
 
 sg_shutdown()           performs global finalizations and releases golbally
-                        aquired resources.
+                        acquired resources.
 
 sg_give_next_adr()      iterates over the set of potentially useful drive 
                         address strings.
@@ -54,7 +54,7 @@ scsi_enumerate_drives() brings all available, not-whitelist-banned, and
                         accessible drives into libburn's list of drives.
 
 sg_dispose_drive()      finalizes adapter specifics of struct burn_drive
-                        on destruction. Releases resources which were aquired
+                        on destruction. Releases resources which were acquired
                         underneath scsi_enumerate_drives().
  
 sg_drive_is_open()      tells wether libburn has the given drive in use.
@@ -78,7 +78,7 @@ burn_os_is_2k_seekrw()  tells whether the given path leads to a file object
 
 burn_os_stdio_capacity()  estimates the emulated media space of stdio-drives.
 
-burn_os_open_track_src()  opens a disk file in a way that allows best
+burn_os_open_track_src()  opens a disk file in a way that offers best
                         throughput with file reading and/or SCSI write command
                         transmission.
 
@@ -149,6 +149,11 @@ Send feedback to libburn-hackers@pykix.org .
 #include <cdio/logging.h>
 #include <cdio/mmc.h>
 
+/* ts B41126 : O_BINARY is needed for Cygwin but undefined elsewhere */
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
 
 /* The waiting time before eventually retrying a failed SCSI command.
    Before each retry wait Libburn_sg_linux_retry_incR longer than with
@@ -390,7 +395,7 @@ int sg_initialize(char msg[1024], int flag)
 
 
 /** Performs global finalization of the SCSI transport adapter and eventually
-    needed operating system facilities. Releases globally aquired resources.
+    needed operating system facilities. Releases globally acquired resources.
     @param flag  unused yet, submit 0
     @return      1 = success, <=0 = failure
 */ 
@@ -524,7 +529,7 @@ ex:;
 
 /** Tells whether libburn has the given drive in use or exclusively reserved.
     If it is "open" then libburn will eventually call sg_release() on it when
-    it is time to give up usage resp. reservation.
+    it is time to give up usage and reservation.
 */
 /** Published as burn_drive.drive_is_open() */
 int sg_drive_is_open(struct burn_drive * d)
@@ -835,7 +840,7 @@ static int freebsd_is_2k_seekrw(char *path, int flag)
 
 
 /* Return 1 if the given path leads to a regular file or a device that can be
-   seeked, read, and possibly written with 2 kB granularity. 
+   fseeked, read, and possibly written with 2 kB granularity. 
 */
 int burn_os_is_2k_seekrw(char *path, int flag)
 {
@@ -895,7 +900,7 @@ int burn_os_stdio_capacity(char *path, off_t write_start, off_t *bytes)
 
 	/* GNU/Linux specific determination of block device size */
 	} else if(S_ISBLK(stbuf.st_mode)) {
-		int open_mode = O_RDONLY, fd;
+		int open_mode = O_RDONLY | O_BINARY, fd;
 		long blocks;
 
 		blocks = *bytes / 512;
@@ -915,7 +920,7 @@ int burn_os_stdio_capacity(char *path, off_t write_start, off_t *bytes)
 	} else if(S_ISCHR(stbuf.st_mode)) {
 		int fd;
 
-		fd = open(path, O_RDONLY);
+		fd = open(path, O_RDONLY | O_BINARY);
 		if (fd == -1)
 			{ret = -2; goto ex;}
 		ret = ioctl(fd, DIOCGMEDIASIZE, &add_size);
@@ -929,7 +934,7 @@ int burn_os_stdio_capacity(char *path, off_t write_start, off_t *bytes)
 #ifdef Libburn_is_on_solariS
 
 	} else if(S_ISBLK(stbuf.st_mode)) {
-		int open_mode = O_RDONLY, fd;
+		int open_mode = O_RDONLY | O_BINARY, fd;
 		
 		fd = open(path, open_mode);
 		if (fd == -1)
@@ -985,7 +990,7 @@ int burn_os_open_track_src(char *path, int open_flags, int flag)
 {
 	int fd;
 
-	fd = open(path, open_flags);
+	fd = open(path, open_flags | O_BINARY);
 	return fd;
 }
 
