@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2012 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2017 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -226,8 +226,6 @@ int burn_write_track_minsize(struct burn_write_opts *o, struct burn_session *s,
 			0x0002011a,
 			LIBDAX_MSGS_SEV_NOTE, LIBDAX_MSGS_PRIO_HIGH, msg,0,0);
 		step = BUFFER_SIZE / 4096; /* shall fit any sector size */
-		if (step <= 0)
-			step = 1;
 		seclen = burn_sector_length(t->mode);
 		if (seclen <= 0)
 			seclen = 2048;
@@ -791,7 +789,7 @@ XXX this is untested :)
 			rem += burn_track_get_shortage(tar[i]);
 
 			/* ts A61101 : I doubt that linking would yield a
-					desireable effect. With TAO it is
+					desirable effect. With TAO it is
 					counterproductive in any way.
 			*/
 			if (o->write_type == BURN_WRITE_TAO)
@@ -1421,7 +1419,7 @@ int burn_precheck_write(struct burn_write_opts *o, struct burn_disc *disc,
 	enum burn_write_types wt;
 	struct burn_drive *d = o->drive;
 	char *msg = NULL, *reason_pt;
-	int no_media = 0, ret, has_cdtext;
+	int no_media = 0, ret, has_cdtext, is_bd_pow = 0;
 
 	reason_pt= reasons;
 	reasons[0] = 0;
@@ -1499,6 +1497,20 @@ int burn_precheck_write(struct burn_write_opts *o, struct burn_disc *disc,
 		   sequential stdio "drive" */
 		if (o->start_byte >= 0)
 			strcat(reasons, "write start address not supported, ");
+
+		is_bd_pow = burn_drive_get_bd_r_pow(d);
+		if (is_bd_pow && !silent) 
+			libdax_msgs_submit(libdax_messenger, d->global_index,
+				0x0002011e,
+				LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
+			   "Unsuitable media detected: BD-R formatted to POW.",
+				0, 0);
+		if (is_bd_pow) {
+			strcat(reasons,
+				"unsuitable media formatting POW detected, ");
+			return 0;
+		}
+
 	} else {
 unsuitable_profile:;
 		msg = calloc(1, 160);
@@ -1690,7 +1702,7 @@ int burn_disc_close_track_dvd_minus_r(struct burn_write_opts *o, int tnum)
 int burn_disc_finalize_dvd_plus_r(struct burn_write_opts *o)
 {
 	struct burn_drive *d = o->drive;
-	char msg[80];
+	char msg[40 + 80]; /* filltext + profile */
 
 	sprintf(msg, "Finalizing %s ...",
 		d->current_profile_text);
@@ -2108,7 +2120,7 @@ int burn_dvd_write_session(struct burn_write_opts *o,
 		libdax_msgs_submit(libdax_messenger, d->global_index,
 				0x00020171,
 				LIBDAX_MSGS_SEV_NOTE, LIBDAX_MSGS_PRIO_HIGH,
-				"Closing BD-R with accidently open session",
+				"Closing BD-R with accidentally open session",
 				0, 0);
  		d->close_track_session(d, 3, 0); /* CLOSE SESSION, 110b */
 		d->state_of_last_session = 3; /* mark as complete session */
@@ -2591,6 +2603,8 @@ int burn_stdio_write(int fd, char *buf, int count, struct burn_drive *d,
 
 	if (d->cancel || count <= 0)
 		return 0;
+	if(d->do_simulate)
+		return 1;
 
 	todo = count;
 	done = 0;
@@ -2621,6 +2635,7 @@ fprintf(stderr, "libburn_DEBUG: write(%d, %lX, %d)\n",
 		d->cancel = 1;
 		ret = 0; goto ex;
 	}
+	ret = 1;
 ex:;
 	BURN_FREE_MEM(msg);
 	return ret;
@@ -2698,7 +2713,7 @@ int burn_stdio_sync_cache(int fd, struct burn_drive *d, int flag)
 			"Invalid file descriptor with stdio pseudo-drive",
 			0, 0);
 		d->cancel = 1;
-		return 0;
+		ret = 0; goto ex;
 	}
 	d->needs_sync_cache = 0;
 	do_fsync = 0;
@@ -2726,7 +2741,7 @@ int burn_stdio_sync_cache(int fd, struct burn_drive *d, int flag)
 			LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
 			msg, errno, 0);
 		d->cancel = 1;
-		return 0;
+		ret = 0; goto ex;
 	}
 	ret = 1;
 ex:;
@@ -2750,16 +2765,15 @@ int burn_stdio_slowdown(struct burn_drive *d, struct timeval *prev_time,
 			 int amount, int flag)
 {
 	struct timeval tnow;
-	struct timezone dummy_tz;
 	double to_wait;
 
 	if (flag & 1) {
-		gettimeofday(prev_time, &dummy_tz);
+		gettimeofday(prev_time, NULL);
 		return 1;
 	}
 	if(d->nominal_write_speed <= 0)
 		return 2;
-	gettimeofday(&tnow, &dummy_tz);
+	gettimeofday(&tnow, NULL);
 	to_wait = ( ((double) amount) / (double) d->nominal_write_speed ) - 
 		(double) ( tnow.tv_sec - prev_time->tv_sec ) -
 		(double) ( tnow.tv_usec - prev_time->tv_usec ) / 1.0e6
@@ -2767,7 +2781,7 @@ int burn_stdio_slowdown(struct burn_drive *d, struct timeval *prev_time,
 	if (to_wait >= 0.0001) {
 		usleep((int) (to_wait * 1000000.0));
 	}
-	gettimeofday(prev_time, &dummy_tz);
+	gettimeofday(prev_time, NULL);
 	return 1;
 }
 
@@ -2797,6 +2811,7 @@ int burn_stdio_write_track(struct burn_write_opts *o, struct burn_session *s,
 		d->write = burn_stdio_mmc_dummy_write;
 	else
 		d->write = burn_stdio_mmc_write;
+	d->do_simulate = o->simulate;
 	d->sync_cache = burn_stdio_mmc_sync_cache;
 
 	burn_stdio_slowdown(d, &prev_time, 0, 1); /* initialize */
@@ -3292,7 +3307,9 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 		rpt += d->buffer->bytes;
 		d->buffer->sectors = chunksize;
 		d->nwa = start;
-		if(d->drive_role == 1) {
+		if(d->do_simulate) {
+			err = 0;
+		} else if(d->drive_role == 1) {
 			err = d->write(d, d->nwa, d->buffer);
 		} else {
 			ret = burn_stdio_write(fd, (char *) d->buffer->data,
@@ -3312,7 +3329,9 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 	if(d->drive_role == 1)
 		d->needs_sync_cache = 1;
 	if(flag & 1) {
-		if(d->drive_role == 1)
+		if(d->do_simulate) {
+			;
+		} else if(d->drive_role == 1)
 			d->sync_cache(d);
 		else
 			burn_stdio_sync_cache(fd, d, 2);
