@@ -1275,6 +1275,8 @@ int Xorriso_findx(struct XorrisO *xorriso, struct FindjoB *job,
  char *name= NULL, *path= NULL, *sfe= NULL;
  char *abs_dir_path_data= NULL, *abs_path= NULL;
 
+ job->depth= depth;
+
  if(xorriso->request_to_abort)
    {ret= 0; goto ex;}
 
@@ -1342,6 +1344,7 @@ int Xorriso_findx(struct XorrisO *xorriso, struct FindjoB *job,
  if(ret==0)
    {ret= 2; goto ex;}
  
+ job->depth++;
  while(!xorriso->request_to_abort) {
    Linkitem_reset_stack(&own_link_stack, link_stack, 0);
    ret= Dirseq_next_adr(dirseq,name,0);
@@ -1403,6 +1406,7 @@ int Xorriso_findx(struct XorrisO *xorriso, struct FindjoB *job,
 
  ret= 1;
 ex:;
+ job->depth= depth;
  Xorriso_free_meM(sfe);
  Xorriso_free_meM(name);
  Xorriso_free_meM(path);
