@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2016 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2017 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -1939,6 +1939,28 @@ ex:;
 }
 
 
+/* Command -use_immed_bit */
+int Xorriso_option_use_immed_bit(struct XorrisO *xorriso, char *mode,
+                                 int flag)
+{
+ int ret;
+
+ if(strncmp(mode, "default", 7) == 0 || mode[0] == 0) {
+   xorriso->use_immed_bit= 0;
+ } else if(strcmp(mode, "on")==0) {
+   xorriso->use_immed_bit= 1;
+ } else if(strcmp(mode, "off")==0) {
+   xorriso->use_immed_bit= -1;
+ } else {
+   sprintf(xorriso->info_text, "-use_immed_bit: unknown mode '%s'", mode);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ }
+ ret= Xorriso_use_immed_bit(xorriso, 0);
+ return(ret);
+}
+
+
 /* Option -use_readline */
 int Xorriso_option_use_readline(struct XorrisO *xorriso, char *mode, int flag)
 {
@@ -1967,7 +1989,7 @@ int Xorriso_option_version(struct XorrisO *xorriso, int flag)
  sprintf(xorriso->result_line,
 "ISO 9660 Rock Ridge filesystem manipulator and CD/DVD/BD burn program\n");
  sprintf(xorriso->result_line+strlen(xorriso->result_line),
-"Copyright (C) 2015, Thomas Schmitt <scdbackup@gmx.net>, libburnia project.\n");
+"Copyright (C) 2017, Thomas Schmitt <scdbackup@gmx.net>, libburnia project.\n");
  Xorriso_result(xorriso, 0);
  sprintf(xorriso->result_line,
          "xorriso version   :  %d.%d.%d%s\n",
@@ -2084,7 +2106,9 @@ int Xorriso_option_volume_date(struct XorrisO *xorriso,
  if(timestring[0] == 0 || strcmp(timestring, "default") == 0 ||
     strcmp(timestring, "overridden") == 0 ){
    t= 0;
- } else if(strcmp(time_type, "uuid") == 0) {
+ } else if(strcmp(time_type, "uuid") == 0 ||
+           (strcmp(time_type, "all_file_dates") == 0 &&
+            strcmp(timestring, "set_to_mtime") == 0)) {
    t= time(NULL); /* Just to have some that is not 0 */
  } else {
    ret= Xorriso_convert_datestring(xorriso, "-volume_date",
@@ -2111,6 +2135,15 @@ int Xorriso_option_volume_date(struct XorrisO *xorriso,
      Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
    }
 
+ } else if(strcmp(time_type, "all_file_dates") == 0) {
+   if(t == 0) {
+     xorriso->all_file_dates[0]= 0;
+     ret= 1; goto ex;
+   }
+   strncpy(xorriso->all_file_dates, timestring,
+           sizeof(xorriso->all_file_dates) - 1);
+   xorriso->all_file_dates[sizeof(xorriso->all_file_dates) - 1]= 0;
+
  } else if(strcmp(time_type, "c") == 0) {
    xorriso->vol_creation_time= t;
  } else if(strcmp(time_type, "m") == 0) {
@@ -2156,11 +2189,14 @@ int Xorriso_option_xattr(struct XorrisO *xorriso, char *mode, int flag)
 {
  int ret;
 
- if(strcmp(mode, "off")==0)
-   xorriso->do_aaip&= ~12;
- else if(strcmp(mode, "on")==0)
+ if(strcmp(mode, "off") == 0) {
+   xorriso->do_aaip&= ~(4 | 8 | 1024);
+ } else if(strcmp(mode, "on") == 0 || strcmp(mode, "user") == 0) {
+   xorriso->do_aaip&= ~1024;
    xorriso->do_aaip|= (4 | 8);
- else {
+ } else if(strcmp(mode, "any") == 0) {
+   xorriso->do_aaip|= (4 | 8 | 1024);
+ } else {
    sprintf(xorriso->info_text, "-xattr: unknown mode '%s'", mode);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
    return(0);
