@@ -55,6 +55,9 @@ int Xorriso_decode_load_adr(struct XorrisO *xorriso, char *cmd,
                             int *entity_code, char entity_id[81],
                             int flag);
 
+int Xorriso_check_thing_len(struct XorrisO *xorriso, char *name, int size,
+                            char *cmd, char *thing, int flag);
+
 int Xorriso_check_name_len(struct XorrisO *xorriso, char *name, int size,
                            char *cmd, int flag);
 
