@@ -2,7 +2,7 @@
 /* xorriso - libisoburn higher level API which creates, loads, manipulates
              and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2016 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2018 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -73,8 +73,8 @@ struct XorrisO;
     @since 0.5.8
 */
 #define Xorriso_header_version_majoR  1
-#define Xorriso_header_version_minoR  4
-#define Xorriso_header_version_micrO  6
+#define Xorriso_header_version_minoR  5
+#define Xorriso_header_version_micrO  0
 
 
 /** If needed: Something like ".pl01" to indicate a bug fix. Normally empty.
@@ -1593,10 +1593,13 @@ int Xorriso_option_follow(struct XorrisO *xorriso, char *mode, int flag);
 /* Command -for_backup is a shortcut for
      Xorriso_option_hardlinks(xorriso, "on", 0);
      Xorriso_option_acl(xorriso, "on", 0);
-     Xorriso_option_xattr(xorriso, "on", 0);
+     Xorriso_option_xattr(xorriso, "any", 0);
      Xorriso_option_md5(xorriso, "on", 0);
 */
 /* @since 0.4.0 */
+/* xattr "any"
+   @since 1.5.0
+*/
 
 /* Command -fs */
 /* @since 0.1.0 */
@@ -1741,6 +1744,8 @@ int Xorriso_option_map(struct XorrisO *xorriso, char *disk_path,
                              1= -compare_l
                              2= -update_l
                              3= -extract_l
+                             4= -update_lxi  @since 1.4.8
+                             5= -update_li   @since 1.4.8
 */
 /* @since 0.2.0 */
 int Xorriso_option_map_l(struct XorrisO *xorriso, int argc, char **argv,
